type 'a t = 'a list
(* hd : 'a list -> 'a *)
let hd = function []   -> failwith "empty list"
                | x::_ -> x
(* tl : 'a list -> 'a list *)
let tl = function []    -> failwith "empty list"
                | _::xs -> xs
(* append : 'a list -> 'a list -> 'a list *)
let rec append xs ys = if xs = [] then ys else hd xs::append (tl xs) ys
(* init : 'a -> int -> 'a list *)
let rec init i l = if l < 1 then [] else i::init i (l-1)
(* take : int -> 'a list -> 'a list *)
let rec take n xs = if n < 1 then [] else match xs with
 | []    -> []
 | x::xs -> x::take (n-1) xs
(* drop : int -> 'a list -> 'a list *)
let rec drop n xs = if n < 1 then xs else match xs with
 | []    -> []
 | _::xs -> drop (n-1) xs
(* split_at : int -> 'a list -> ('a list * 'a list) *)
let split_at n xs = (take n xs,drop n xs)
(* map : ('a -> 'b) -> 'a list -> 'b list *)
let rec map f = function []    -> []
                       | x::xs -> f x::map f xs
(* foldr : ('a -> 'b -> 'a) -> 'a -> 'a -> 'b list *)
let rec foldr f xs b = match xs with
 | []    -> b
 | x::xs -> f x (foldr f xs b)
(* *)
